@@ -62,6 +62,7 @@ import java.lang.reflect.Modifier;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
+import java.util.HashMap;
 import java.util.Iterator;
 import java.util.LinkedHashMap;
 import java.util.LinkedHashSet;
@@ -69,6 +70,9 @@ import java.util.LinkedList;
 import java.util.List;
 import java.util.Map;
 
+import static org.codehaus.groovy.ast.tools.GeneralUtils.returnS;
+import static org.codehaus.groovy.ast.tools.GeneralUtils.stmt;
+import static org.codehaus.groovy.ast.tools.GeneralUtils.varX;
 import static org.codehaus.groovy.ast.tools.GenericsUtils.correctToGenericsSpecRecurse;
 
 /**
@@ -141,7 +145,7 @@ public abstract class TraitComposer {
             String name = methodNode.getName();
             Parameter[] helperMethodParams = methodNode.getParameters();
             boolean isAbstract = methodNode.isAbstract();
-            if (!isAbstract && helperMethodParams.length > 0 && ((methodNode.getModifiers() & Opcodes.ACC_STATIC) == Opcodes.ACC_STATIC) && !name.contains("$")) {
+            if (!isAbstract && helperMethodParams.length > 0 && ((methodNode.getModifiers() & Opcodes.ACC_STATIC) == Opcodes.ACC_STATIC) && (!name.contains("$") || (methodNode.getModifiers() & Opcodes.ACC_SYNTHETIC) == 0)) {
                 ArgumentListExpression argList = new ArgumentListExpression();
                 argList.addExpression(new VariableExpression("this"));
                 Parameter[] origParams = new Parameter[helperMethodParams.length - 1];
@@ -149,7 +153,7 @@ public abstract class TraitComposer {
                 System.arraycopy(methodNode.getParameters(), 1, params, 0, params.length);
                 Map<String,ClassNode> methodGenericsSpec = new LinkedHashMap<String, ClassNode>(genericsSpec);
                 MethodNode originalMethod = trait.getMethod(name, params);
-                // Original method may be null in case of a private method
+                // Original method may be null for the case of private or static methods
                 if (originalMethod!=null) {
                     methodGenericsSpec = GenericsUtils.addMethodGenerics(originalMethod, methodGenericsSpec);
                 }
@@ -207,6 +211,7 @@ public abstract class TraitComposer {
                     String operation = methodNode.getName().substring(suffixIdx + 1);
                     boolean getter = "get".equals(operation);
                     ClassNode returnType = correctToGenericsSpecRecurse(genericsSpec, methodNode.getReturnType());
+                    int fieldMods = 0;
                     int isStatic = 0;
                     boolean publicField = true;
                     FieldNode helperField = fieldHelperClassNode.getField(Traits.FIELD_PREFIX + Traits.PUBLIC_FIELD_PREFIX + fieldName);
@@ -222,15 +227,30 @@ public abstract class TraitComposer {
                             publicField = false;
                             helperField = fieldHelperClassNode.getField(Traits.STATIC_FIELD_PREFIX+Traits.PRIVATE_FIELD_PREFIX +fieldName);
                         }
+                        fieldMods = fieldMods | Opcodes.ACC_STATIC;
                         isStatic = Opcodes.ACC_STATIC;
                     }
+                    if (helperField == null) {
+                        fieldMods = 0;
+                        isStatic = 0;
+                        for (Integer mod : Traits.FIELD_PREFIXES) {
+                            helperField = fieldHelperClassNode.getField(String.format("$0x%04x", mod) + fieldName);
+                            if (helperField != null) {
+                                if ((mod & Opcodes.ACC_STATIC) != 0) isStatic = Opcodes.ACC_STATIC;
+                                fieldMods = fieldMods | mod;
+                                break;
+                            }
+                        }
+                    } else {
+                        fieldMods = fieldMods | (publicField?Opcodes.ACC_PUBLIC:Opcodes.ACC_PRIVATE);
+                    }
                     if (getter) {
                         // add field
                         if (helperField!=null) {
                             List<AnnotationNode> copied = new LinkedList<AnnotationNode>();
                             List<AnnotationNode> notCopied = new LinkedList<AnnotationNode>();
                             GeneralUtils.copyAnnotatedNodeAnnotations(helperField, copied, notCopied);
-                            FieldNode fieldNode = cNode.addField(fieldName, (publicField?Opcodes.ACC_PUBLIC:Opcodes.ACC_PRIVATE) | isStatic, returnType, null);
+                            FieldNode fieldNode = cNode.addField(fieldName, fieldMods, returnType, (fieldMods & Opcodes.ACC_FINAL) == 0 ? null : helperField.getInitialExpression());
                             fieldNode.addAnnotations(copied);
                         }
                     }
@@ -243,28 +263,30 @@ public abstract class TraitComposer {
                         newParams = new Parameter[]{new Parameter(fixedType, "val")};
                     }
 
-                    Expression fieldExpr = new VariableExpression(cNode.getField(fieldName));
+                    Expression fieldExpr = varX(cNode.getField(fieldName));
                     Statement body =
-                            getter ? new ReturnStatement(fieldExpr) :
-                                    new ExpressionStatement(
+                            getter ? returnS(fieldExpr) :
+                                    stmt(
                                             new BinaryExpression(
                                                     fieldExpr,
                                                     Token.newSymbol(Types.EQUAL, 0, 0),
-                                                    new VariableExpression(newParams[0])
+                                                    varX(newParams[0])
                                             )
                                     );
-                    MethodNode impl = new MethodNode(
-                            methodNode.getName(),
-                            Opcodes.ACC_PUBLIC | isStatic,
-                            returnType,
-                            newParams,
-                            ClassNode.EMPTY_ARRAY,
-                            body
-                    );
-                    AnnotationNode an = new AnnotationNode(COMPILESTATIC_CLASSNODE);
-                    impl.addAnnotation(an);
-                    cNode.addTransform(StaticCompileTransformation.class, an);
-                    cNode.addMethod(impl);
+                    if (getter || (fieldMods & Opcodes.ACC_FINAL) == 0) {
+                        MethodNode impl = new MethodNode(
+                                methodNode.getName(),
+                                Opcodes.ACC_PUBLIC | isStatic,
+                                returnType,
+                                newParams,
+                                ClassNode.EMPTY_ARRAY,
+                                body
+                        );
+                        AnnotationNode an = new AnnotationNode(COMPILESTATIC_CLASSNODE);
+                        impl.addAnnotation(an);
+                        cNode.addTransform(StaticCompileTransformation.class, an);
+                        cNode.addMethod(impl);
+                    }
                 }
             }
         }
@@ -292,7 +314,8 @@ public abstract class TraitComposer {
 
         ClassNode[] exceptionNodes = correctToGenericsSpecRecurse(genericsSpec, copyExceptions(helperMethod.getExceptions()));
         ClassNode fixedReturnType = correctToGenericsSpecRecurse(genericsSpec, helperMethod.getReturnType());
-        Expression forwardExpression = genericsSpec.isEmpty()?mce:new CastExpression(fixedReturnType,mce);
+        boolean noCastRequired = genericsSpec.isEmpty() || fixedReturnType.getName().equals(ClassHelper.VOID_TYPE.getName());
+        Expression forwardExpression = noCastRequired ? mce : new CastExpression(fixedReturnType,mce);
         int access = helperMethod.getModifiers();
         // we could rely on the first parameter name ($static$self) but that information is not
         // guaranteed to be always present
@@ -319,10 +342,20 @@ public abstract class TraitComposer {
         if (!copied.isEmpty()) {
             forwarder.addAnnotations(copied);
         }
-        if (originalMethod!=null) {
+        if (originalMethod != null) {
             GenericsType[] newGt = GenericsUtils.applyGenericsContextToPlaceHolders(genericsSpec, originalMethod.getGenericsTypes());
             newGt = removeNonPlaceHolders(newGt);
             forwarder.setGenericsTypes(newGt);
+        } else {
+            // null indicates a static method which may still need generics correction
+            GenericsType[] genericsTypes = helperMethod.getGenericsTypes();
+            if (genericsTypes != null) {
+                Map<String,ClassNode> methodSpec = new HashMap<String, ClassNode>();
+                methodSpec = GenericsUtils.addMethodGenerics(helperMethod,methodSpec);
+
+                GenericsType[] newGt = GenericsUtils.applyGenericsContextToPlaceHolders(methodSpec, helperMethod.getGenericsTypes());
+                forwarder.setGenericsTypes(newGt);
+            }
         }
         // add a helper annotation indicating that it is a bridge method
         AnnotationNode bridgeAnnotation = new AnnotationNode(Traits.TRAITBRIDGE_CLASSNODE);
@@ -352,7 +385,7 @@ public abstract class TraitComposer {
             }
         }
         if (!modified) return oldTypes;
-        if (l.size()==0) return null;
+        if (l.isEmpty()) return null;
         return l.toArray(new GenericsType[l.size()]);
     }
 
@@ -486,34 +519,6 @@ public abstract class TraitComposer {
         return false;
     }
 
-    /**
-     * An utility method which tries to find a method with default implementation (in the Java 8 semantics).
-     * @param cNode a class node
-     * @param name the name of the method
-     * @param params the parameters of the method
-     * @return a method node corresponding to a default method if it exists
-     */
-    private static MethodNode findDefaultMethodFromInterface(final ClassNode cNode, final String name, final Parameter[] params) {
-        if (cNode == null) {
-            return null;
-        }
-        if (cNode.isInterface()) {
-            MethodNode method = cNode.getMethod(name, params);
-            if (method!=null && !method.isAbstract()) {
-                // this is a Java 8 only behavior!
-                return method;
-            }
-        }
-        ClassNode[] interfaces = cNode.getInterfaces();
-        for (ClassNode anInterface : interfaces) {
-            MethodNode res = findDefaultMethodFromInterface(anInterface, name, params);
-            if (res!=null) {
-                return res;
-            }
-        }
-        return findDefaultMethodFromInterface(cNode.getSuperClass(), name, params);
-    }
-
     private static boolean isExistingProperty(final String methodName, final ClassNode cNode, final Parameter[] params) {
         String propertyName = methodName;
         boolean getter = false;
