@@ -61,7 +61,7 @@ public class GroovyDocToolTest extends GroovyTestCase {
 
         xmlToolForTests = new GroovyDocTool(
                 new FileSystemResourceManager("src"), // template storage
-                new String[] {"src/test/groovy", "../../src/test"}, // source file dirs
+                new String[] {"src/test/groovy", "src/test/resources", "../../src/test"}, // source file dirs
                 new String[]{TEMPLATES_DIR + "/topLevel/rootDocStructuredData.xml"},
                 new String[]{TEMPLATES_DIR + "/packageLevel/packageDocStructuredData.xml"},
                 new String[]{TEMPLATES_DIR + "/classLevel/classDocStructuredData.xml"},
@@ -314,6 +314,18 @@ public class GroovyDocToolTest extends GroovyTestCase {
         assertTrue("innerClassMethod found in: \"" + classWithAnonymousInnerClassDoc + "\"", !classWithAnonymousInnerClassDoc.contains("innerClassMethod"));
     }
 
+    public void testJavaClassWithDiamondOperator() throws Exception {
+        List<String> srcList = new ArrayList<String>();
+        String base = "org/codehaus/groovy/tools/groovydoc/testfiles/JavaClassWithDiamond";
+        srcList.add(base + ".java");
+        xmlToolForTests.add(srcList);
+        MockOutputTool output = new MockOutputTool();
+        xmlToolForTests.renderToOutput(output, MOCK_DIR);
+        String doc = output.getText(MOCK_DIR + "/" + base + ".html");
+        assertNotNull("No GroovyDoc found for " + base, doc);
+        assertTrue("stringList not found in: \"" + doc + "\"", doc.contains("stringList"));
+    }
+
     public void testVisibilityPublic() throws Exception {
         Properties props = new Properties();
         props.put("publicScope", "true");
@@ -585,6 +597,31 @@ public class GroovyDocToolTest extends GroovyTestCase {
         assertEquals("There has to be a reference to class ArrayList", "ArrayList", m.group(2));
     }
 
+    public void testScript() throws Exception {
+        List<String> srcList = new ArrayList<String>();
+        srcList.add("org/codehaus/groovy/tools/groovydoc/testfiles/Script.groovy");
+        xmlTool.add(srcList);
+
+        MockOutputTool output = new MockOutputTool();
+        xmlTool.renderToOutput(output, MOCK_DIR);
+        String scriptDoc = output.getText(MOCK_DIR + "/org/codehaus/groovy/tools/groovydoc/testfiles/Script.html");
+        assertTrue("There should be a reference to method sayHello", containsTagWithName(scriptDoc, "method", "sayHello"));
+        assertTrue(scriptDoc, scriptDoc.contains("Use this to say Hello"));
+
+        assertTrue("There should be a reference to method sayGoodbye", containsTagWithName(scriptDoc, "method", "sayGoodbye"));
+        assertTrue(scriptDoc, scriptDoc.contains("Use this to bid farewell"));
+
+        assertTrue("There should be a reference to property instanceProp", containsTagWithName(scriptDoc, "property", "instanceProp"));
+
+        assertTrue("There should be a reference to field staticField", containsTagWithName(scriptDoc, "field", "staticField"));
+
+        assertFalse("Script local variables should not appear in groovydoc output", scriptDoc.contains("localVar"));
+    }
+
+    private boolean containsTagWithName(String text, String tagname, String name) {
+        return text.matches("(?s).*<"+ tagname + "[^>]* name=\""+ name + "\".*");
+    }
+
     private GroovyClassDoc getGroovyClassDocByName(GroovyRootDoc root, String name) {
         GroovyClassDoc[] classes = root.classes();
 
